<?php
/**
 * @file
 * Theme functions for RoyalSlider.
 */

/**
 * Implements hook_preprocess_HOOK().
 *
 * Preprocess variables for the RoyalSlider template.
 */
function royalslider_preprocess_royalslider(&$variables) {
  // Load optionset.
  if ($optionset = ctools_export_crud_load('royalslider_optionset', $variables['settings']['optionset'])) {
    // Don't need this in settings array anymore.
    unset($variables['settings']['optionset']);

    // Make sure we have a unique ID.
    $ids = &drupal_static(__FUNCTION__, array());
    $id = $variables['settings']['id'];
    $i = 0;

    while (in_array($id, $ids)) {
      $id .= ++$i;
    }

    // Add wrapper attributes.
    $variables['attributes_array']['class'][] = 'royalSlider';
    $variables['attributes_array']['id'] = $id;

    // Setup skin & fallback to default.
    $skins = royalslider_skins();
    $skin_class = isset($skins[$optionset->skin]['class'])
      ? $skins[$optionset->skin]['class']
      : $skins['default']['class'];

    $variables['attributes_array']['class'][] = $skin_class;

    // Copy items array for preprocessing.
    $items_processed = array();

    foreach ($variables['items'] as $item) {
      // Parse item.
      $doc = new DOMDocument();
      $doc->loadHTML($item);

      // Save the tag to alter.
      // This is necessary because enabling the preloader will replace all
      // "img" tags with "a" tags.
      $target_tag = 'img';

      foreach ($doc->getElementsByTagName('img') as $img) {
        $alt = $img->getAttribute('alt');
        $title = $img->getAttribute('title');

        // Find image description.
        // Voodoo nested ternary statement ftw.
        $description = !empty($alt)
          ? $alt
          : !empty($title) ?
            $title
            : '';

        // If we're using the preloader, replace image tags with a link tag.
        if ($optionset->options['usePreloader']) {
          // Rewrite images as links to enable RoyalSlider's lazy-loading and also
          // add descriptions.
          $link = $doc->createElement('a');
          $link->setAttribute('href', $img->getAttribute('src'));
          $link->setAttribute('class', 'rsImg');
          $desc = $doc->createTextNode($description);
          $link->appendChild($desc);

          // Replace image with link.
          $img->parentNode->replaceChild($link, $img);
          $target_tag = 'a';
        }
        // If we're not, just add the "rsImg" class to the image tags.
        else {
          $old_classes = explode(' ', $img->getAttribute('class'));
          $new_classes = array('rsImg');

          foreach ($old_classes as $class) {
            $new_classes[] = $class;
          }

          $img->setAttribute('class', implode(' ', $new_classes));
        }

        // Apply image style to main image, if applicable.
        if (!empty($optionset->imagestyle_normal)) {
          $image_style = $optionset->imagestyle_normal;
          // In this case, we're replacing the main image, not adding a new
          // attribute.
          $url_attribute = ($target_tag === 'a') ? 'href' : 'src';
          _royalslider_apply_image_style($doc, $target_tag, $image_style, $url_attribute);
        }

        // Explicitly set thumbnail images and apply image styles if applicable.
        if ($optionset->options['controlNavigation'] === 'thumbnails') {
          $image_style = $optionset->imagestyle_thumbnail;
          _royalslider_apply_image_style($doc, $target_tag, $image_style, 'data-rsTmb');
        }

        // Explicitly set fullscreen images and apply image styles if applicable.
        if ($optionset->options['fullscreen']['enabled']) {
          $image_style = $optionset->imagestyle_fullscreen;
          _royalslider_apply_image_style($doc, $target_tag, $image_style, 'data-rsBigImg');
        }

        // Automatically calculate main image dimensions.
        // @TODO remove the first FALSE when we're ready to test this.
        if (FALSE && $optionset->options['drupalAutoSetDimensions']) {
          foreach ($doc->getElementsByTagName($target_tag) as $tag) {
            $classes = $tag->getAttribute('class');

            // Make sure it's been preprocessed.
            if (strpos($classes, 'rsImg') !== FALSE) {
              $url = $tag->getAttribute($url_attribute);
              $size = getimagesize($url);
              include_once drupal_get_path('module', 'devel') . '/krumo/class.krumo.php';
              krumo($url, $size);die;
            }
          }
        }
      }

      // Give modules a last chance to alter the DomDocument before it gets
      // serialized.
      drupal_alter('royalslider_slide', $doc, $id, $optionset);

      // Back to HTML.
      $items_processed[] = filter_dom_serialize($doc);
    }

    if (!empty($items_processed)) {
      // Add div wrappers around each item.
      $variables['contents'] = '<div>' . implode('</div><div>', $items_processed) . '</div>';

      // Add resources.
      royalslider_add($id, $optionset);

      // We're done!
      return;
    }
  }

  // If we got this far, something went wrong.
  // @TODO throw exception? error message? log error? lolcat? fail silently?
}

/**
 * Helper function to apply image styles and apply attributes for image paths.
 *
 * @param DomDocument
 *   The DomDocument object we're manipulating.
 * @param string
 *   The HTML tag we want to target images in.
 * @param string
 *   The image style we want to apply to the images.
 * @param string
 *   The attribute name we want to set on the tag with the new image path.
 *
 * @see royalslider_preprocess_royalslider()
 */
function _royalslider_apply_image_style(DomDocument &$doc, $target_tag, $image_style, $attribute) {
  // Check for remote stream wrapper.
  static $remote_stream_wrapper;
  if (!isset($remote_stream_wrapper)) {
    $remote_stream_wrapper = module_exists('remote_stream_wrapper');
  }

  // Set attribute for finding the image path.
  $url_attribute = ($target_tag === 'a') ? 'href' : 'src';

  foreach ($doc->getElementsByTagName($target_tag) as $tag) {
    $classes = $tag->getAttribute('class');

    // Make sure it's been preprocessed.
    if (strpos($classes, 'rsImg') !== FALSE) {
      // @TODO grab relative image path from current absolute path so it doesn't
      // get treated as a remote stream wrapper by mistake.
      // @TODO infer original image from style
      $url = $tag->getAttribute($url_attribute);

      if (!empty($image_style)) {
        if ($remote_stream_wrapper) {
          $new_path = file_is_scheme_remote(file_uri_scheme($url))
            ? remote_stream_wrapper_image_style_path($image_style, $url)
            : image_style_url($image_style, $url);
        }
        else {
          $new_path = image_style_url($image_style, $url);
        }
      }
      else {
        $new_path = $url;
      }

      $new_path = file_create_url($new_path);

      $tag->setAttribute($attribute, $new_path);
    }
  }
}
